#
# WiKicker
#
#  DbFile.t
#
#
# Copyright (C) 2003,2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: DbFile.t,v 1.10 2005/05/10 12:56:03 naney Exp $
#
#

use strict;
use Test::More tests => 74;
use File::Path;
use File::Spec;

use WiKicker::DB::File;
use WiKicker::TrackBack::EntryMap;
use WiKicker::TrackBack::Entry;
use Storable;

use vars qw($dbdir);


BEGIN {
  $dbdir = File::Spec->catfile('t', 'tmp', 'WiKicker-test-db');
  rmtree([$dbdir], 0);
}

END {
  rmtree([$dbdir], 0);
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    new with no parameter
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  eval {
    my $db = WiKicker::DB::File->new;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
  }
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    create db
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);
  # last_modified_file_name
  is($db->last_modified_file_name,
     File::Spec->catfile($dbdir, 'last-modified.txt'));
  is($db->lock_file_name('test'),
     File::Spec->catfile($dbdir, 'test.lock'));

  ok(! -e $db->last_modified_file_name);

  # db_last_modified_time
  is($db->db_last_modified_time, 0);

  # exists_live_page
  ok(! $db->exists_live_page);
  ok(! $db->exists_live_page('key'));

  # exists_backup_page
  ok(! $db->exists_backup_page);
  ok(! $db->exists_backup_page('key'));

  # page
  $db->shared_lock_pages;
  ok(! defined $db->page('key'));
  $db->unlock_pages;

  # page (without key)
  eval {
    $db->page;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
  }

  # page_revision_no
  eval {
    $db->page_revision_no;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
  }

  $db->shared_lock_pages;
  is($db->page_revision_no('key'), 0);
  $db->unlock_pages;

  # page_list
  ok(eq_array([ sort $db->page_list ], []));

  # last_modified
  is($db->last_modified('key'), 0);

  # page_subject
  ok(! defined $db->page_subject('key'));

  # page_author
  ok(! defined $db->page_author('key'));
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    reopen
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);
  # last_modified_file_name
  is($db->last_modified_file_name,
     File::Spec->catfile($dbdir, 'last-modified.txt'));
  ok(! -e $db->last_modified_file_name);

  # db_last_modified_time
  is($db->db_last_modified_time, 0);

  # exists_live_page
  ok(! $db->exists_live_page);
  ok(! $db->exists_live_page('key'));

  # exists_backup_page
  ok(! $db->exists_backup_page);
  ok(! $db->exists_backup_page('key'));

  # page
  $db->shared_lock_pages;
  ok(! defined $db->page('key'));
  $db->unlock_pages;

  # page (without key)
  eval {
    $db->page;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
  }

  # page_revision_no;
  $db->shared_lock_pages;
  is($db->page_revision_no('key'), '0');
  $db->unlock_pages;

  # page_list
  ok(eq_array([ sort $db->page_list ], []));

  # last_modified
  is($db->last_modified('key'), 0);

  # page_subject
  ok(! defined $db->page_subject('key'));

  # page_author
  ok(! defined $db->page_author('key'));
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    page_revision_no() (without lock, without key)
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  # without lock
  eval {
    $db->page_revision_no('key');
  };
  if ($@) {
    pass();
  }
  else {
    fail();
    diag("lock check");
  }

  # without key
  $db->shared_lock_pages;
  eval {
    $db->page_revision_no;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
  }
  $db->unlock_pages;
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    get_revision (without parameter);
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  # without lock
  eval {
    $db->get_revision;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
    diag("lock check");
  }
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    get_head_revision (without parameter);
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  # without lock
  eval {
    $db->get_head_revision;
  };
  if ($@) {
    pass();
  }
  else {
    fail();
    diag("lock check");
  }
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    write page
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  my $now = time;
  $db->exclusive_lock_pages;
  $db->page('key', 'value');
  is($db->page('key'), 'value');
  $db->unlock_pages;

  # exists_live_page
  ok($db->exists_live_page('key'));

  # last_modified_file_name
  ok(-e $db->last_modified_file_name);

  # db_last_modified_time
  my $db_last_modified_time = $db->db_last_modified_time;
  ok($now <= $db_last_modified_time && $db_last_modified_time <= $now + 5)
    || diag("now($now) != last_modified($db_last_modified_time)");

  $db->shared_lock_pages;
  # page_revision_no;
  is($db->page_revision_no('key'), '1.1');
  # get_revision;
  is($db->get_revision('key', '1.1'), 'value');
  # get_head_revision
  is($db->get_head_revision('key'), 'value');
  $db->unlock_pages;

  # page_list
  ok(eq_array([ sort $db->page_list ], [ 'key' ]));

  # page_name_list
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'key' ]));
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'key' ])); # read cache

  # last_modofied
  my $last_modified = $db->last_modified('key');
  ok($now <= $last_modified && $last_modified <= $now + 5)
    || diag("now($now) != last_modified($last_modified)");


  # page_subject
  is($db->page_subject('key'), '');

  # page_author
  is($db->page_author('key'), '');
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    set information
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  $db->exclusive_lock_pages;
  $db->information('key', 'author', 'naney');
  $db->shared_lock_pages;
  is($db->information('key', 'author'), 'naney');
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    delete page
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  # cache page_name_list object
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'key' ]));

  ok(!$db->exists_live_page('deletetest'));
  $db->exclusive_lock_pages;
  # write
  $db->page('deletetest', 'value');

  # check
  is($db->page('deletetest'), 'value');
  ok($db->exists_live_page('deletetest'));
  ok(eq_array([ sort $db->page_list ], [ 'deletetest', 'key' ]));
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'deletetest', 'key' ]))
    || diag join(', ', sort @{$db->page_name_list->list});

  # delete
  $db->delete_page('deletetest');
  ok(!$db->exists_live_page('deletetest'));
  $db->unlock_pages;

  # page_list
  ok(eq_array([ sort $db->page_list ], [ 'key' ]));
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'key' ]))
    || diag join(', ', sort @{$db->page_name_list->list});
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    write with information
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  my $now = time;
  $db->exclusive_lock_pages;
  $db->page('key2', 'page-value', 'author-value', 'subject-value');
  is($db->page('key2'), 'page-value');
  $db->unlock_pages;

  # exists_live_page
  ok($db->exists_live_page('key2'));

  # last_modified_file_name
  ok(-e $db->last_modified_file_name);

  # db_last_modified_time
  my $db_last_modified_time = $db->db_last_modified_time;
  ok($now <= $db_last_modified_time && $db_last_modified_time <= $now + 5)
    || diag("now($now) != last_modified($db_last_modified_time)");

  $db->shared_lock_pages;
  # page_revision_no;
  is($db->page_revision_no('key2'), '1.1');
  # get_revision;
  is($db->get_revision('key2', '1.1'), 'page-value');
  # get_head_revision
  is($db->get_head_revision('key2'), 'page-value');
  $db->unlock_pages;

  # page_list
  ok(eq_array([ sort $db->page_list ], [ 'key', 'key2' ]));

  # page_name_list
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'key', 'key2' ]));
  ok(eq_array([ sort @{$db->page_name_list->list} ], [ 'key', 'key2' ])); # read cache

  $db->shared_lock_pages;
  # last_modofied
  my $last_modified = $db->last_modified('key');
  ok($now <= $last_modified && $last_modified <= $now + 5);

  # page_subject
  is($db->page_subject('key2'), 'subject-value');

  # page_author
  is($db->page_author('key2'), 'author-value');
  $db->unlock_pages;
}

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#    set information
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
{
  my $db = WiKicker::DB::File->new($dbdir);

  $db->exclusive_lock_pages;
  $db->information('key', 'author', 'naney');
  $db->shared_lock_pages;
  is($db->information('key', 'author'), 'naney');
}

# cache
{
  my $db = WiKicker::DB::File->new($dbdir);
  $db->cache('link', 'key', 'a b');
  is($db->cache('link', 'key'), 'a b');
}

# cache (TrackBack)
{
  my $db = WiKicker::DB::File->new($dbdir);

  my $entry_map = WiKicker::TrackBack::EntryMap->new;
  my $entry = WiKicker::TrackBack::Entry->new;
  $entry->url('http://www.naney.org/');
  $entry_map->set($entry);

  $db->cache('trackback', 'key', Storable::freeze($entry_map));

  my $entry_map2 = Storable::thaw($db->cache('trackback', 'key'));
  my $entry2 = $entry_map2->get('http://www.naney.org/');
  is($entry2->url, 'http://www.naney.org/');
}
