#
# WiKicker
#
#  WRIFactory.t
#
#
# Copyright (C) 2003,2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: WRIFactory.t,v 1.18 2005/03/20 10:44:26 naney Exp $
#
#

use strict;
use Test;

BEGIN { plan tests => 104 }

use WiKicker::WRI::Factory;
use WiKicker::WRI::Resolver;
use WiKicker::WikiPage::Env;
use Log::Log4perl qw(:easy);
BEGIN {
  mkdir('t/tmp', 0755);
}
Log::Log4perl->easy_init( {level => $DEBUG,
                           file => '>t/tmp/WRIFactory.log',
                           layout => '%m%n',
                          }
                          );

my $factory = WiKicker::WRI::Factory->new;

my $resolver = WiKicker::WRI::Resolver->new;
$resolver->amazon_id('wwwnaneyorg-22');
my $wri_env = WiKicker::WikiPage::Env->new(resolver => $resolver,
                                           wikicker_url => '/wiki',
                                           wri_factory => $factory,
                                          );

# http
{
  my $wri = $factory->create_wri('http://www.naney.org/');
  ok(ref $wri eq 'WiKicker::WRI::URL');
  ok($wri->scheme eq 'http');
  ok($wri->to_html('', $wri_env) eq
     '<a href="http://www.naney.org/">http://www.naney.org/</a>');
}

# http (image)
{
  my $wri = $factory->create_wri('http://www.naney.org/tmp.jpg');
  ok(ref $wri eq 'WiKicker::WRI::URL');
  ok($wri->scheme eq 'http');
  ok($wri->to_html('', $wri_env) =~ m|http://www.naney.org/tmp.jpg|);
}

# page name
{
  my $wri = $factory->create_wri('PageName');
  ok(ref $wri eq 'WiKicker::WRI::PWRN');
  ok($wri->page_name eq 'PageName');
  ok($wri->fragment_identifier eq '');
}

# page name with :
{
  my $wri = $factory->create_wri('WiKicker\:\:WRI');
  ok(ref $wri eq 'WiKicker::WRI::PWRN');
  ok($wri->page_name eq 'WiKicker::WRI');
  ok($wri->fragment_identifier eq '');
}


# page-subject:
{
  my $wri = $factory->create_wri('page-subject:PageName');
  ok(ref $wri eq 'WiKicker::WRI::PageSubject');
}

# fragment identifier only
{
  my $wri = $factory->create_wri('#comment');
  ok(ref $wri eq 'WiKicker::WRI::PWRN');
  ok($wri->page_name eq '');
  ok($wri->fragment_identifier eq 'comment');
}

# inter wiki definition
{
  my $wri = $factory->create_wri('interwiki:name:def');
  ok($wri->scheme eq 'interwiki');
}

# inter wiki definition (def includes :)
{
  my $wri = $factory->create_wri('interwiki:name:file:////a');
  ok($wri->scheme eq 'interwiki');
}

# isbn
{
  my $wri = $factory->create_wri('ISBN:0-201-61641-6');
  ok(ref $wri eq 'WiKicker::WRI::ISBN');
  ok($wri->scheme eq 'ISBN');
  ok($wri->rest eq '0-201-61641-6');
  my $urn = $wri->canonical;
  ok(ref $urn eq 'WiKicker::WRI::URN');
  ok($urn->nid eq 'isbn');
  ok($urn->nss eq '0-201-61641-6');
  ok($wri->get_link_url eq '');
  ok($wri->get_link_url($wri_env)
    eq 'http://www.amazon.co.jp/exec/obidos/ASIN/0201616416/wwwnaneyorg-22/ref=nosim');
}

# isbn
{
  my $wri = $factory->create_wri('ISBN0-201-61641-6');
  ok(ref $wri eq 'WiKicker::WRI::SWRN');
  my $urn = $wri->canonical;
  ok($urn->nid eq 'isbn');
  ok($urn->nss eq '0-201-61641-6');
  ok($wri->get_link_url($wri_env)
    eq 'http://www.amazon.co.jp/exec/obidos/ASIN/0201616416/wwwnaneyorg-22/ref=nosim');
}

# isbn
{
  my $wri = $factory->create_wri('image:ISBN:0-201-61641-6');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ISBN:0-201-61641-6');
  ok($wri->get_link_url($wri_env)
    eq 'http://www.amazon.co.jp/exec/obidos/ASIN/0201616416/wwwnaneyorg-22/ref=nosim');
  ok($wri->get_image_url($wri_env)
     eq 'http://images-jp.amazon.com/images/P/0201616416.09.MZZZZZZZ.jpg');
}

# isbn
{
  my $wri = $factory->create_wri('image:ISBN:0070072728');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ISBN:0070072728');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/0070072728/wwwnaneyorg-22/ref=nosim');
  ok($wri->get_image_url($wri_env)
     eq 'http://images-jp.amazon.com/images/P/0070072728.01.MZZZZZZZ.jpg');
}

# isbn
{
  my $wri = $factory->create_wri('image:ISBN:0596003285');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ISBN:0596003285');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/0596003285/wwwnaneyorg-22/ref=nosim');
 ok($wri->get_image_url($wri_env)
  eq 'http://images-jp.amazon.com/images/P/0596003285.01.MZZZZZZZ.jpg');
}

# isbn
{
  my $wri = $factory->create_wri('image:ISBN:0767905385');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ISBN:0767905385');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/0767905385/wwwnaneyorg-22/ref=nosim');
 ok($wri->get_image_url($wri_env)
  eq 'http://images-jp.amazon.com/images/P/0767905385.01.MZZZZZZZ.jpg');
}

# no isbn
{
  my $wri = $factory->create_wri('ISBN');
  ok(ref $wri eq 'WiKicker::WRI::PWRN');
}

# asin
{
  my $wri = $factory->create_wri('image:ASIN:B000065EA0');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ASIN:B000065EA0');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/B000065EA0/wwwnaneyorg-22/ref=nosim');
  ok($wri->get_image_url($wri_env)
     eq 'http://images-jp.amazon.com/images/P/B000065EA0.09.MZZZZZZZ.jpg');
}

# asin
{
  my $wri = $factory->create_wri('image:ASIN:B0000645I7');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ASIN:B0000645I7');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/B0000645I7/wwwnaneyorg-22/ref=nosim');
  ok($wri->get_image_url($wri_env)
     eq 'http://images-jp.amazon.com/images/P/B0000645I7.01.MZZZZZZZ.jpg');
}

# asin
{
  my $wri = $factory->create_wri('image:ASIN:B00008W2OO');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ASIN:B00008W2OO');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/B00008W2OO/wwwnaneyorg-22/ref=nosim');
  ok($wri->get_image_url($wri_env)
     eq 'http://images-jp.amazon.com/images/P/B00008W2OO.01.MZZZZZZZ.jpg');
}

# asin
{
  my $wri = $factory->create_wri('image:ASIN:B00004R7VU');
  ok($wri->scheme eq 'image');
  ok($wri->rest eq 'ASIN:B00004R7VU');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.amazon.co.jp/exec/obidos/ASIN/B00004R7VU/wwwnaneyorg-22/ref=nosim');
  ok($wri->get_image_url($wri_env)
     eq 'http://images-jp.amazon.com/images/P/B00004R7VU.01.MZZZZZZZ.jpg');
}


# flickr
{
  my $wri = $factory->create_wri('flickr:mauriago:http://photos4.flickr.com/4567089_66cb434558_d.jpg');
  ok($wri->scheme eq 'flickr');
  ok($wri->rest eq 'mauriago:http://photos4.flickr.com/4567089_66cb434558_d.jpg');
  ok($wri->get_link_url($wri_env)
     eq 'http://www.flickr.com/photos/mauriago/4567089/');
  ok($wri->get_image_url($wri_env)
     eq 'http://photos4.flickr.com/4567089_66cb434558_d.jpg');
}


# rfc
{
  my $wri = $factory->create_wri('rfc1234');
  ok(ref $wri eq 'WiKicker::WRI::SWRN');
  ok($wri->get_link_url
     eq 'http://www.ietf.org/rfc/rfc1234.txt');
}

# rfc
{
  my $wri = $factory->create_wri('RFC1234');
  ok(ref $wri eq 'WiKicker::WRI::SWRN');
  ok($wri->get_link_url
     eq 'http://www.ietf.org/rfc/rfc1234.txt');
}

# rfc
{
  my $wri = $factory->create_wri('rfc:1234');
  ok(ref $wri eq 'WiKicker::WRI::Rfc');
  ok($wri->get_link_url
     eq 'http://www.ietf.org/rfc/rfc1234.txt');
}

# rfc
{
  my $wri = $factory->create_wri('RFC:1234');
  ok(ref $wri eq 'WiKicker::WRI::Rfc');
  ok($wri->get_link_url
     eq 'http://www.ietf.org/rfc/rfc1234.txt');
}

# no rfc
{
  my $wri = $factory->create_wri('RFC');
  ok(ref $wri eq 'WiKicker::WRI::PWRN');
}

# gmtime
{
  my $wri = $factory->create_wri('gmtime:0');
  ok($wri->scheme eq 'gmtime');
  ok($wri->rest eq '0');
}

# gmtime
{
  my $wri = $factory->create_wri('gmtime:now');
  ok($wri->scheme eq 'gmtime');
  ok($wri->rest eq 'now');
}

# href
{
  my $wri = $factory->create_wri('href:http://www.naney.org/tmp.jpg');
  ok(ref $wri eq 'WiKicker::WRI::Href');
  ok($wri->get_link_url
     eq 'http://www.naney.org/tmp.jpg');
  ok($wri->get_image_url($wri_env) eq '');
}

# search
{
  my $wri = $factory->create_wri('search:WikiPage');
  ok(ref $wri eq 'WiKicker::WRI::Search');
  ok($wri->to_html('', $wri_env) =~ m|^<a href="/wiki\?search=WikiPage".*>search:WikiPage</a>$|);
}


# empty
{
  my $wri = $factory->create_wri('');
  ok(ref $wri eq 'WiKicker::WRI::SWRN');
  ok($wri->get_link_url eq '');
  ok($wri->get_image_url($wri_env) eq '');
}

# get_scheme
{
  my ($scheme, $rest) = WiKicker::WRI::Factory::get_scheme('abc');
  ok($scheme eq '');
  ok($rest eq 'abc');

  ($scheme, $rest) = WiKicker::WRI::Factory::get_scheme('http:...');
  ok($scheme eq 'http');
  ok($rest eq '...');

  ($scheme, $rest) = WiKicker::WRI::Factory::get_scheme('a:b:c');
  ok($scheme eq 'a');
  ok($rest eq 'b:c');

  ($scheme, $rest) = WiKicker::WRI::Factory::get_scheme('a\:b:cde');
  ok($scheme eq 'a\:b');
  ok($rest eq 'cde');

  ($scheme, $rest) = WiKicker::WRI::Factory::get_scheme('limage:http://www.naney.org/tmp.jpg');
  ok($scheme eq 'limage');
  ok($rest eq 'http://www.naney.org/tmp.jpg');
}

# with user_rwrn_map_object
{
  my $factory = WiKicker::WRI::Factory->new('user_rwrn_map_object'
                                            => RwrnMapMock->new);

  my $wri1 = $factory->create_wri('user-defined:value');
  ok(ref($wri1), 'UserDefinedRwrn');

  # again
  my $wri2 = $factory->create_wri('user-defined:value');
  ok(ref($wri2), 'UserDefinedRwrn');

  # no class
  my $wri3 = $factory->create_wri('user-defined-no-class:value');
  ok(ref($wri3), 'WiKicker::WRI::UWRN');

  open(LOG, 't/tmp/WRIFactory.log') || die "Can't open log file: $!";
  my @logs = <LOG>;
  close(LOG);

  ok(grep { /Can't load RWRN module UserDefinedRwrn:/ } @logs);
  ok(grep { /Can't load RWRN module UserDefinedNoClassRwrn:/ } @logs);
  ok(grep { /Can't create user defined RWRN object UserDefinedNoClassRwrn:/ } @logs);
}

package RwrnMapMock;
use strict;

sub new {
  my $class = shift;

  return bless {}, $class;
}

sub user_scheme_to_class_name {
  my ($self, $scheme) = @_;

  if ($scheme eq 'user-defined') {
    return 'UserDefinedRwrn';
  }
  if ($scheme eq 'user-defined-no-class') {
    return 'UserDefinedNoClassRwrn';
  }

  return undef;
}

package UserDefinedRwrn;

use WiKicker::WRI::RWRN;
use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::WRI::RWRN);
}
